<?php
// +----------------------------------------------------------------------+
// | Author:  Evgeny Leontev <eleontev@ezactive.com>                      |
// | Copyright (c) 2005 Evgeny Leontev                                    |
// +----------------------------------------------------------------------+
// | This source file is free software; you can redistribute it and/or    |
// | modify it under the terms of the GNU Lesser General Public           |
// | License as published by the Free Software Foundation; either         |
// | version 2.1 of the License, or (at your option) any later version.   |
// |                                                                      |
// | This source file is distributed in the hope that it will be useful,  |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of       |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    |
// | Lesser General Public License for more details.                      |
// +----------------------------------------------------------------------+


class MyDir
{
	
	var $skip_dirs = array();
	var $skip_files = array();
	var $allowed_extensions = array();
	
	

   /**
	* Read $dirname, return multi array with files and dirs on success
	*
	* @param       string   $dirname    The directory to read
	* @return      bool     Returns true on success, false on failure
	*/
	function & getFilesDirs($dirname = '.'){
		
		if ($handle = opendir($dirname)) {
			while(false !== ($entry = readdir($handle))){
				if($entry == '.' || $entry == '..' || in_array($entry, $this->skip_dirs)) { continue; }
				
				$path = $dirname.'/'.$entry;
				if(is_dir($path))  { $output[$entry] = $this->getFilesDirs($path); }
				if(is_file($path)) {
					if(in_array($path, $this->skip_files)) { continue; }
					
					// skip not allowed extensions
					if($this->allowed_extensions) {
						if(!in_array(substr($path, -4), $this->allowed_extensions)) { continue; }
					}
					
					$output[] = $entry;
				}
			}
			
			return isset($output) ? $output : array();
			closedir($handle);
		
		} else {
			return false;
		}
	}	
	
	
	
   /**
	* Delete a file, or a folder and its contents
	*
	* @param       string   $dirname    The directory to delete
	* @return      bool     Returns true on success, false on failure
	*/
	function removeFilesDirs($dirname = '.') {
		
		if (is_file($dirname)) { return unlink($dirname); }
		
		$dir = dir($dirname);
		while (false !== $entry = $dir->read()) {
			if ($entry == '.' || $entry == '..' || in_array($entry, $this->skip_dirs)) { continue; }
			
			// Deep delete directories  
			$path = $dirname . '/' . $entry;
			if (is_dir($path)) { $this->removeFilesDirs($path); }
			else               { unlink($path); }
		}
		
		$dir->close();
		return rmdir($dirname);
	}
	

   /**
	* Sets directories to skip (not parse)
	*
	* @param       mixed   $extension    directory name
	* @return      void     
	*/
	function setSkipDirs($dirname) {
		foreach($this->_toArray(func_get_args()) as $v) {
			$this->skip_dirs[] = $dirname;
		}
	}
	

   /**
	* Sets files to skip (not parse)
	* for filename use top_dir/sub_dir/filename.php 
	*
	* @param       mixed   $extension    filename
	* @return      void     
	*/
	function setSkipFiles($filename) {
		foreach($this->_toArray(func_get_args()) as $v) {
			$this->skip_files[] = $filename;
		}
	}

	
   /**
	* Define allowed extensions
	*
	* @param       mixed   $extension    allowed extesions
	* @return      void     
	*/ 
	function setAllowedExtensions($extension) {	
		foreach($this->_toArray(func_get_args()) as $v) {
			$this->allowed_extensions[] = '.' . str_replace('.', '', $v);
		}
	}
	
	
	function _toArray($val) {
		if(!is_string($val[0])) { return $val[0]; }
		return $val;
	}
}


/*
$d = &new MyDir;
$d->setSkipDirs('.svn', 'cvs');
$d->setAllowedExtensions('php', 'ini');

$a = &$d->getFilesDirs('e:/www/localhost/a_repository/eleontev');

//echo "<pre>"; print_r($a); echo "</pre>";


//require_once 'adodb.inc.php';


function multiArrayToString($arr, $glue=',') { 
	foreach($arr as $k => $v) {
		if(is_array($v)) { 
			$arr[$k] = multiArrayToString($v, $glue); 
		}
	}
	return $arr;  
} 


echo "<pre>"; print_r(multiArrayToString($a)); echo "</pre>";
*/
?>