<?php
// +---------------------------------------------------------------------------+
// | This file is part of the KnowledgebasePublisher package                   |
// | KnowledgebasePublisher - web based knowledgebase publisher tool           |
// |                                                                           |
// | Author:  Evgeny Leontev <eleontev@gmail.com>                              |
// | Copyright (c) 2005 Evgeny Leontev                                         |
// |                                                                           |
// | For the full copyright and license information, please view the LICENSE   |
// | file that was distributed with this source code.                          |
// +---------------------------------------------------------------------------+

require_once APP_MODULE_DIR . 'knowledgebase/category/inc/KBCategoryModel.php';


class KBEntryModel extends AppModel
{
	var $tbl_pref_custom = 'kb_';
	var $tables = array('table'=>'entry', 'entry', 'category', 'entry_to_category', 
	                    'comment', 'rating', 'attachment_to_entry', 'attachment');
	
	
	var $attachment_dir;
	var $attachment_path;
	var $use_attachment = false;
	
	
	function getById($record_id) {
		$sql = "SELECT e.*, UNIX_TIMESTAMP(e.date_posted) AS ts
		FROM {$this->tbl->entry} e
		WHERE e.id = ?";
		
		//echo "<pre>"; print_r($sql); echo "</pre>";
		$result =& $this->db->Execute($sql, $record_id) or die(db_error($sql));
		return $result->FetchRow();
	}

	
	function getRecordsSql() {
		
		$sql = "
		SELECT 
			e.*,
			IF(e.body != '', e.active, 'not') AS active,
			CONCAT(SUBSTRING_INDEX(e.body,' ',20), '...') AS body, 
			cat.id AS category_id,
			cat.name AS category_name,
			#cat.private AS category_private,
			r.votes AS votes,
			(r.rate/r.votes) AS rating,
			COUNT(DISTINCT c.id) AS comment_num,
			COUNT(DISTINCT a.attachment_id) AS attachment_num,
			UNIX_TIMESTAMP(e.date_posted) AS ts
			
		FROM 
			{$this->tbl->entry} e,
			{$this->tbl->category} cat,
			{$this->tbl->entry_to_category} e_to_cat
		
		LEFT JOIN {$this->tbl->comment} c ON e.id = c.entry_id
		LEFT JOIN {$this->tbl->rating} r ON e.id = r.entry_id
		LEFT JOIN {$this->tbl->attachment_to_entry} a ON e.id = a.entry_id		
		
		WHERE 1
			AND e.id = e_to_cat.entry_id
			AND cat.id = e_to_cat.category_id
			AND {$this->sql_params}
		GROUP BY e.id
		{$this->sql_params_order}";
		
		//echo "<pre>"; print_r($sql); echo "</pre>";
		return $sql;
	}		
	
	
	// for page by page
	function getCountRecordsSql() {
		$sql = "SELECT COUNT(DISTINCT(e.id)) 
		FROM 
			{$this->tbl->entry} e,
			{$this->tbl->category} cat,
			{$this->tbl->entry_to_category} e_to_cat		
		WHERE e.id = e_to_cat.entry_id
		AND cat.id = e_to_cat.category_id
		AND {$this->sql_params}";
		return $sql;
	}	
	
	
	// CATEGORIES // ---------------------------
	
	// return categories array for entry
	function &getCategoryById($record_id) {
		$sql = "SELECT category_id AS id, category_id FROM {$this->tbl->entry_to_category} 
		WHERE entry_id IN ($record_id)";
		$result =& $this->db->Execute($sql) or die(db_error($sql));
	
		return $result->GetAssoc();
	}
	
	
	// get array with categories names to use in list records
	function &getCategoryByIds($record_id) {
		
		$sql = "
		SELECT 
			ec.entry_id, 
			c.name 
		FROM 
			{$this->tbl->category} c, 
			{$this->tbl->entry_to_category} ec
		WHERE 1
			AND ec.entry_id IN ($record_id) 
			AND c.id = ec.category_id";
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		while($row = $result->FetchRow()){
			$data[$row['entry_id']][] = $row['name'];
		}
		
		return $data;
	}
	
	
	// range for form select
	function &getCategorySelectRange() {
		$cat = new KBCategoryModel();
		$rows = &$cat->getSelectRecords();
		return $cat->getSelectRange($rows);
	}
	
	
	// ATACHMENT // -------------------------
	
	// get array with attachments filenames
	function &getAttachmentById($record_id) {
		
		$sql = "
		SELECT 
			f.id, 
			f.filename 
		FROM 
			{$this->tbl->file} f, 
			{$this->tbl->attachment_to_entry} ae
		WHERE 1
			AND ae.entry_id IN ($record_id) 
			AND ae.attachment_id = f.id";
			
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return $result->GetAssoc();
	}
	
	
	function getAttachmentByAttachmentId($ids) {
		$sql = "SELECT f.id, a.filename 
		FROM {$this->tbl->file} f
		WHERE f.id IN ($ids)";
			
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return $result->GetAssoc();
		
	}
	
	
	
	// ACTIONS // ---------------------
	
	function addEntry(&$obj) {
		return $this->add($obj);
	}	
	
	
	function updateEntry(&$obj) {		
		$this->update($obj);
	}
	
	
	function saveEntryToCategory(&$obj, $record_id) {
		
		require_once 'eleontev/SQL/MultiInsert.php';
		
		$cat = $obj->getCategory();
		$cat = (is_array($cat)) ? $cat : array($cat);
		foreach($cat as $k => $v) {
			$categories[] = array($v);
		}
		
		$sql = MultiInsert::get("INSERT {$this->tbl->entry_to_category} (category_id, entry_id) 
		                         VALUES ?", $categories, array($record_id));
		
		return $this->db->Execute($sql) or die(db_error($sql));
	}
	
	
	function saveAttachmentToEntry(&$obj, $record_id) {
		
		if(!$obj->getAttachment()) { return; }
		
		require_once 'eleontev/SQL/MultiInsert.php';
		
		$cat = $obj->getAttachment();
		$cat = (is_array($cat)) ? $cat : array($cat);
		foreach($cat as $k => $v) {
			$attachment[] = array($v);
		}
			
		$sql = MultiInsert::get("INSERT IGNORE {$this->tbl->attachment_to_entry} (attachment_id, entry_id) 
		                        VALUES ?", $attachment, array($record_id));
		
		return $this->db->Execute($sql) or die(db_error($sql));
	}

	
	function save(&$obj) {
		
		if(!$obj->get('id')) {
			
			$id = $this->addEntry($obj);
			$this->saveEntryToCategory($obj, $id);
			
			if($this->use_attachment) {
				$this->saveAttachmentToEntry($obj, $id);
			}
			
			
		} else {
			
			$id = $obj->get('id');
			$this->updateEntry($obj);
			$this->deleteEntryToCategory($id);
			$this->saveEntryToCategory($obj, $id);
			
			if($this->use_attachment) {
				$this->deleteAttachmentToEntry($id);
				$this->saveAttachmentToEntry($obj, $id);
			}
		}
		
		return $id;
	}
	
	
	// OTHER // ------------------------------
	
	// if check priv is different for model so reassign 	
	function checkPriv(&$priv, $action, $record_id) {
		
		$priv->setOwnParam(sprintf("user_id=%d", $priv->user_id));
		
		$sql = "SELECT 1 FROM {$this->tbl->table} 
		WHERE id = '{$record_id}'
		AND user_id = {$priv->user_id}";
		$priv->setOwnSql($sql);
		$priv->check($action);
		
		// set sql to select own records
		$this->setSqlParams('AND ' . $priv->getOwnParam());
	}
	
	
	// DELETE RELATED // --------------------- 
	
	function deleteEntries($record_id) {
		$sql = "DELETE FROM {$this->tbl->entry} WHERE id IN ({$record_id})";
		return $this->db->Execute($sql) or die(db_error($sql));
	}
	
	function deleteRating($record_id) {
		$sql = "DELETE FROM {$this->tbl->rating} WHERE entry_id IN ({$record_id})";
		return $this->db->Execute($sql) or die(db_error($sql));
	}
	
	function deleteComments($record_id) {
		$sql = "DELETE FROM {$this->tbl->comment} WHERE entry_id IN ({$record_id})";
		return $this->db->Execute($sql) or die(db_error($sql));
	}
	
	function deleteEntryToCategory($record_id) {
		$sql = "DELETE FROM {$this->tbl->entry_to_category} WHERE entry_id IN ({$record_id})";
		return $this->db->Execute($sql) or die(db_error($sql));
	}
	
	function deleteAttachmentToEntry($record_id) {
		$sql = "DELETE FROM {$this->tbl->attachment_to_entry} WHERE entry_id IN ({$record_id})";
		return $this->db->Execute($sql) or die(db_error($sql));
	}
	
	
	function delete($record_id) {
		
		// convert to string 1,2,3... to use in IN()
		$record_id = $this->idToString($record_id);
		
		$this->deleteEntries($record_id);
		$this->deleteRating($record_id);
		$this->deleteComments($record_id);
		$this->deleteEntryToCategory($record_id);
		
		if($this->use_attachment) {
			$this->deleteAttachmentToEntry($record_id);
		}
	}
}
?>
