<?php
// +---------------------------------------------------------------------------+
// | This file is part of the KnowledgebasePublisher package                   |
// | KnowledgebasePublisher - web based knowledgebase publisher tool           |
// |                                                                           |
// | Author:  Evgeny Leontev <eleontev@gmail.com>                              |
// | Copyright (c) 2005 Evgeny Leontev                                         |
// |                                                                           |
// | For the full copyright and license information, please view the LICENSE   |
// | file that was distributed with this source code.                          |
// +---------------------------------------------------------------------------+

class UserModel extends AppModel
{

	var $tables = array('table'=>'user', 'user', 'priv', 'priv_name');
	var $crypt_func = 'md5';
		

	// if check priv is different for model so reassign 	
	function checkPriv(&$priv, $action, $record_id) {
		
		$priv->setOwnParam(sprintf("id=%d", $priv->user_id));
		
		$sql = "SELECT 1 FROM {$this->tbl->user} 
		WHERE id = '{$record_id}'
		AND id = {$priv->user_id}";
		$priv->setOwnSql($sql);
		$priv->check($action);
		
		$this->setSqlParams('AND ' . $priv->getOwnParam());
	}
	
	
	function & getById($record_id) {
		$sql = "SELECT * FROM {$this->tbl->user} WHERE id = ?";
		$result =& $this->db->Execute($sql, $record_id) or die(db_error($sql));
		return $result->FetchRow();
	}
	
	
	function getRecordsSql() { 
		//SQL_CALC_FOUND_ROWS
		$sql = "SELECT * FROM {$this->tbl->user} WHERE {$this->sql_params}";
		return $sql;
	}
	
	
	function & getRecords($limit, $offset) {
		$sql = $this->getRecordsSql();
		$result =& $this->db->SelectLimit($sql, $limit, $offset) or die(db_error($sql));
		return $result->GetArray();
	}
	
	
	function save(&$obj, $grantor = false) {
		
		if(!$obj->get('id')) {
			
			$user_id = $this->add($obj);
			
			if($obj->is_admin) {
				$this->addPriv($obj, $user_id);
			}
			
		} else {
			
			$this->update($obj);
			$id = $obj->get('id');
			
			if($obj->is_admin) {
				$this->deletePriv($id);
				$this->addPriv($obj, $id, $grantor);
			}
		}
	}	
	
	
	function deleteUser($record_id) {
		$sql = "DELETE FROM {$this->tbl->user} WHERE id IN ($record_id)";
		return $this->db->Execute($sql) or die(db_error($sql));
	}
	
	
	function delete($record_id) {
		
		// convert to string 1,2,3... to use in IN()
		$record_id = $this->idToString($record_id);
		
		$this->deleteUser($record_id);
		$this->deletePriv($record_id);
	}
	
	
	// return true if we already have such username false otherwise
	function isUsernameExists($username, $id = false) {
		
		$cond = ($id) ? "id != '$id'" : "1=1"; 
		
		$sql = "SELECT 1 FROM {$this->tbl->user} WHERE username = '$username' AND $cond";
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return (bool) ($result->Fields(1));
	}
	
	
	// return true if we already have such username false otherwise
	function isEmailExists($email, $id = false) {
		
		$cond = ($id) ? "email != '$email'" : "1=1"; 
		
		$sql = "SELECT 1 FROM {$this->tbl->user} WHERE email = '$email' AND $cond";
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return (bool) ($result->Fields(1));
	}
	
	
/*
	function isOneAdminUser($user_id) {
		$sql = "SELECT COUNT(*) AS num FROM {$this->tbl->user} 
		WHERE id != '{$user_id}'
		AND active = 1";
		$result =& $this->db->Execute($sql, $record_id) or die(db_error($sql));
		return $result->Fields('num');
	}
*/
	
	
	// PRIV // -----------------
	
	// return user permission array
	function & getPrivById($record_id) {
		$sql = "SELECT id, priv_name_id FROM {$this->tbl->priv} 
		WHERE user_id = '{$record_id}'";
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return $result->GetAssoc();
	}

	
	// in list records
	function & getPrivByIds($record_id) {		
		$sql = "SELECT user_id, priv_name_id 
		FROM {$this->tbl->priv} WHERE user_id IN ($record_id)";
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		while($row = $result->FetchRow()){
			$data[$row['user_id']][] = $row['priv_name_id'];
		}
		
		return $data;
	}	
	
	
	// get array with user_id to user_priv_name use in list records
/*
	function & getPrivByIds($record_id) {
		
		$sql = "SELECT p.user_id, n.name 
		FROM {$this->tbl->priv} p, {$this->tbl->priv_name} n
		WHERE p.user_id IN ($record_id) AND n.id = p.priv_name_id";
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		while($row = $result->FetchRow()){
			$data[$row['user_id']][] = $row['name'];
		}
		
		return $data;
	}

	function getPrivDescription() {
		$sql = "SELECT n.name, n.description 
		FROM {$this->tbl->priv_name} n 
		WHERE active = 1";
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return $result->GetArray();		
	}
*/	
	
	// return all privieges to generate select
	function getAllPriv($cond = '1=1') {
		$sql = "SELECT n.id, n.name FROM {$this->tbl->priv_name} n WHERE $cond";
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return ($result) ? $result->GetAssoc() : array();
	}
	
	
	function deletePriv($record_id) {
		$sql = "DELETE FROM {$this->tbl->priv} WHERE user_id IN ($record_id)";
		return $this->db->Execute($sql) or die(db_error($sql));
	}
	
	
	function addPriv(&$obj, $record_id, $grantor_id = false) {
		
		require_once 'eleontev/SQL/MultiInsert.php';
		
		$priv = $obj->getPriv();
		$priv = (is_array($priv)) ? $priv : array($priv);
		foreach($priv as $k => $v) {
			$privs[] = array($v);
		}
		
		$sql = MultiInsert::get("INSERT {$this->tbl->priv} (priv_name_id, id, user_id, grantor, timestamp) 
		                         VALUES ?", $privs, array(NULL, $record_id, $grantor_id, 'NOW()'));
		
		return $this->db->Execute($sql) or die(db_error($sql));
	}
}
?>